@@ -1,4 +1,5 @@
 import socket
+import sys
 import json
 import gymnasium as gym
 import numpy as np
@@ -185,10 +186,10 @@ class TestEnv(gym.Env):
         # Step again if testing agent is not next
         if not self.was_term and not self.was_trunc and self.agent not in active_agents:
             obs = self.obss[self.env.unwrapped.agent_selection]
-            if (obs is not None
-                and isinstance(obs, dict)
-                and obs and "action_mask" in obs):
-                action = np.random.choice(obs["action_mask"].nonzero()[0])
+            if (isinstance(obs, dict) and "action_mask" in obs):
+                legal_mask = obs["action_mask"]
+                legal_actions = legal_mask.nonzero()[0]
+                action = np.random.choice(legal_actions)
             else:
                 action = self.env.action_space(self.env.unwrapped.agent_selection).sample()
             return self.step(action)
@@ -244,15 +245,17 @@ class TournamentConnection:
         # Initialize class variables
         self.botname = botname
         self.ip_address = ip
-        self.port = port
+        self.port = int(port)
         self.key = key
         self.main_connection = None  # Connection to tournament server
         self.tournament_completed = False
         self.debug = debug
-        # Grab all available games        
-        if self.debug: print("Connecting to matchmaker for following games: ", available_games)
+        # Grab all available games
+        if self.debug:
+            print("Connecting to matchmaker for following games: ", available_games)
         if available_games == ["__all__"]:
-            if self.debug: print("TESTING ALL GAMES")
+            if self.debug:
+                print("TESTING ALL GAMES")
             available_games = list(all_environments.keys())
         # Initialize available games
         # Test agent in every game
@@ -291,6 +294,7 @@ class TournamentConnection:
             print("Not enough players to start tournament.")
             raise err
         spinner.succeed()
+
         if self.debug:
             print(ready_data)
         send_json(self.main_connection, {"type": "ready"})
@@ -337,8 +341,13 @@ class TournamentConnection:
         except TimeoutError as err:
             print("Failed to connect to matchmaker.")
             raise err
+        except json.decoder.JSONDecodeError as err:
+            print("Server returned an invalid response")
+            raise err
 
         # Handle connection errors
+        if init_data["type"] == "malformed_creds":
+            raise RuntimeError("Credentials were formatted incorrectly")
         if init_data["type"] == "bad_creds":
             raise RuntimeError("server did not accept credentials")
         if init_data["type"] == "bad_client_version":
@@ -354,17 +363,18 @@ class TournamentConnection:
         if init_data["complete"]:
             self.tournament_completed = True
 
-    # TODO: Implement terminate signal for tournament and receive it here
     def next_match(self):
         if self.current_match > 0:
             spinner = Halo(text='Waiting for next round', text_color='cyan', color='green', spinner='dots')
             spinner.start()
+
         # Create tournament server connection if it does not already exist
         if self.main_connection is None:
             try:
                 self._setup_main_connection()
             except Exception as e:
                 raise e
+
         if self.current_match > 0:
             spinner.succeed()
 
